@@ -1071,8 +1071,9 @@ struct ath_softc {
 #endif
 
 #ifdef CPTCFG_ATH9K_HWRNG
+	struct hwrng rng_ops;
 	u32 rng_last;
-	struct task_struct *rng_task;
+	char rng_name[sizeof("ath9k_65535")];
 #endif
 };
 
